use strictures 1;
use Test::More qw(no_plan);

use Devel::Dwarn;
use Data::Query::Renderer::SQL::Naive;
use Data::Query::ExprHelpers qw(perl_scalar_value);

BEGIN { require 't/expr.include' }

my $rend = Data::Query::Renderer::SQL::Naive->new({ quote_chars => [ "'" ] });

sub expr_sql_is (&;@) {
  my $sub = shift;
  @_
    ? is_deeply($rend->render(_mk_expr($sub)), @_)
    : ::Dwarn($rend->render(_mk_expr($sub)));
}

expr_sql_is { $_->foo }
  [ 'foo' ],
  "Simple identifier -> SQL";

expr_sql_is { $_->group }
  [ q{'group'} ],
  "Simple identifier needing quoting -> SQL";

expr_sql_is { $_->foo->group }
  [ q{foo.'group'} ],
  "Complex identifier -> SQL";

expr_sql_is { $_->foo == 1 }
  [
    "foo = ?",
    {
      subtype => {
        Perl => "Scalar"
      },
      type => "Value",
      value => 1
    }
  ],
  "Simple expression -> SQL";

expr_sql_is { ($_->foo == 1) & ($_->bar eq "foo") }
  [
    "( foo = ? AND bar = ? )",
    {
      subtype => {
        Perl => "Scalar"
      },
      type => "Value",
      value => 1
    },
    {
      subtype => {
        Perl => "Scalar"
      },
      type => "Value",
      value => "foo"
    }
  ],
  "Compound expression -> SQL";


expr_sql_is { ($_->foo == 1) & ($_->bar eq "foo") & ($_->baz > 3) }
  [
    "( foo = ? AND bar = ? AND baz > ? )",
    {
      subtype => {
        Perl => "Scalar"
      },
      type => "Value",
      value => 1
    },
    {
      subtype => {
        Perl => "Scalar"
      },
      type => "Value",
      value => "foo"
    },
    {
      subtype => {
        Perl => "Scalar"
      },
      type => "Value",
      value => 3
    }
  ],
  "Flatten expression ok";

expr_sql_is { !$_->foo }
  [ "NOT foo" ],
  "Unary expression ok";

expr_sql_is { SELECT { $_->foo } }
  [ "SELECT foo" ],
  "Simple identifier";

expr_sql_is { SELECT { $_->foo, 1 } }
  # the extra space here is a little icky but Naive's _flatten_structure
  # will need rewriting to fix it - commit bits available if you do it first
  [ "SELECT foo , ?", perl_scalar_value(1) ],
  "Identifier and literal";
